/*
 * main_basic.c - a simplified main procedure that relies upon a ersver to do
 * anything smart. Just be a dumb interface to a display, keypad, coin mech
 * and snacks.
 */

#include "display_basic.h"
#include "keypad.h"
#include "chime.h"
#include "coinmech.h"
#include "motors.h"
#include "comm.h"
#include "vend.h"

void motor_reply(char* slotptr, u8 code) {
	/* returns a message of the form MXYY - X is return code, YY is motor */
	wait_for_tx_free();
	tx_buffer[0] = 'M';
	tx_buffer[1] = code + '0';
	tx_buffer[2] = *slotptr;
	tx_buffer[3] = *(slotptr+1);
	tx_buffer[4] = '\n';
	tx_buffer[5] = 0;
	send_packet();
}

void dispense_something() {
	/* process a message VXX in msg_buf where XX is motor number */
	u8 slot;

	if ((msg_buf[1] < '0') || (msg_buf[1] > '9') ||
		(msg_buf[2] < '0') || (msg_buf[2] > '9')) {
		msg_buf[1] = msg_buf[2] = '0';
		motor_reply((char*)&msg_buf[1], MOTOR_NOSLOT);
		return;
	}

	slot = (msg_buf[1] - '0') * 10;
	slot += msg_buf[2] - '0';

	motor_reply((char*)&msg_buf[1], dispense_motor(slot));
}

void write_to_display() {
	/* process a message in the form DXXXXXXXXXXX to send to display */
	u8 i;
	char buf[10];
	for (i = 0; i < 10; i++)
		if (msg_buf[i+1])
			buf[i] = msg_buf[i+1];
		else
			break;

	for (; i < 10; i++) /* pad the rest out with spaces */
		buf[i] = ' ';

	set_msg(buf);
	send_ack();
}

void send_balance() {
	wait_for_tx_free();
	tx_buffer[0] = 'C';
	tx_buffer[1] = (coin_value/10000)%10;
	tx_buffer[2] = (coin_value/1000)%10;
	tx_buffer[3] = (coin_value/100)%10;
	tx_buffer[4] = (coin_value/10)%10;
	tx_buffer[5] = coin_value%10;
	tx_buffer[6] = '\n';
	tx_buffer[7] = 0;
	send_packet();
}

void give_change() {
	u16 cost;

	if ((msg_buf[1] < '0') || (msg_buf[1] > '9') ||
		(msg_buf[2] < '0') || (msg_buf[2] > '9') ||
		(msg_buf[3] < '0') || (msg_buf[3] > '9') ||
		(msg_buf[4] < '0') || (msg_buf[4] > '9') ||
		(msg_buf[5] < '0') || (msg_buf[5] > '9')) {
		send_nack();
	}
	cost = msg_buf[1] - '0';
	cost *= 10; cost = msg_buf[2] - '0';
	cost *= 10; cost = msg_buf[3] - '0';
	cost *= 10; cost = msg_buf[4] - '0';
	cost *= 10; cost = msg_buf[5] - '0';

	coin_cost(cost);
	send_ack();
} 

void send_keypress(u8 key) {
	/* send a packet of the form KX with X being the key, or R for reset */
	wait_for_tx_free();
	tx_buffer[0] = 'K';
	if (key == KEY_RESET)
		tx_buffer[1] = 'R';
	else
		tx_buffer[1] = (key%10)+'0';
	tx_buffer[2] = '\n';
	tx_buffer[3] = 0;
	send_packet();
}

void send_door_msg(bool open) {
	wait_for_tx_free();
	tx_buffer[0] = 'D';
	tx_buffer[1] = open?'1':'0';
	tx_buffer[2] = '\n';
	tx_buffer[3] = 0;
	send_packet();
}

void do_chime() {
	chime_start();
	send_ack();
}

int main() {
	u16 last_coin_value;
	bool last_door_open;

	chime_start();
	
	misc_output = 0;
	changer_output = 0x7f;
	_io_ports[M6811_PORTA] = 0xc8;
	_io_ports[M6811_DDRD] = 0x3e;
	_io_ports[M6811_SPCR] = 0x12;

	unlock(); /* enable interrupts */

	display_init();
	comm_init();
	coinmech_init();

	last_coin_value = 0;
	last_door_open = 0;
	
	while(1) {
		if (door_open() != last_door_open) {
			last_door_open = door_open();
			send_door_msg(last_door_open);
		}

		if (rx_queue_state) {
			switch (msg_buf[0]) {
				case 'V':
					dispense_something();
					break;
				case 'D':
					write_to_display();
					break;
				case 'B': 
					do_chime();
					break;
				case 'U':
					send_balance();
					break;
				case 'G':
					give_change();
					break;
				default:
				/* shrug */
					send_nack();
					break;
			}
			msg_clr();
		}

		keypad_read();
		if (keypad_pressed()) {
			send_keypress(last_key);
		}

		if (coin_value != last_coin_value) {
			send_balance();
			last_coin_value = coin_value;
		}
	}
}

